------------------------------------------------------------------------------
--
-- Filename:                monitor.vhd
-- Inhalt:                  Beispiel fuer passive Prozeduren, Prozesse
--                          und Assertions
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder 
-- Letzte Aenderung:        03.02.94
--
------------------------------------------------------------------------------



------------------------------------------------------------------------------
--
-- anything_one: Verwendung passiver Prozeduren
--
------------------------------------------------------------------------------

ENTITY anything_one IS

  PORT (sig_a, sig_b : IN bit; sig_c : OUT bit ) ;

  PROCEDURE monitoring (SIGNAL   a        : IN bit;
                        CONSTANT sig_name : IN string) IS
  BEGIN
    ASSERT false REPORT "Event on signal " & sig_name SEVERITY note ;
  END monitoring ;

BEGIN

  mon_sig_a : monitoring (sig_a, "anything_one:sig_a") ;
   
END anything_one ;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF anything_one IS

BEGIN

  mon_sig_b : monitoring (a => sig_b, sig_name => "anything_one:sig_b") ;

  sig_c <= sig_a AND NOT sig_b ;

END behavioral ;



------------------------------------------------------------------------------
--
-- anything_two: Verwendung passiver Prozesse
--
------------------------------------------------------------------------------

ENTITY anything_two IS

  PORT (sig_a, sig_b : IN bit; sig_c : OUT bit ) ;

BEGIN

  mon_sig_a : PROCESS (sig_a)
  BEGIN
    ASSERT false REPORT "Event on signal anything_two:sig_a" SEVERITY note ;
  END PROCESS ;
  
END anything_two ;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF anything_two IS

BEGIN

  mon_sig_b : PROCESS (sig_b)
  BEGIN
    ASSERT false REPORT "Event on signal anything_two:sig_b" SEVERITY note ;
  END PROCESS ;

  sig_c <= sig_a AND NOT sig_b ;

END behavioral ;



------------------------------------------------------------------------------
--
-- anything_three: Verwendung von Assertions
--
------------------------------------------------------------------------------

ENTITY anything_three IS

  PORT (sig_a, sig_b : IN bit; sig_c : OUT bit ) ;

BEGIN

  mon_sig_a : ASSERT (sig_a'EVENT = false) 
              REPORT "Event on signal anything_three:sig_a" SEVERITY note ;
 
END anything_three ;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF anything_three IS

BEGIN

  mon_sig_b : ASSERT (sig_b'EVENT = false) 
              REPORT "Event on signal anything_three:sig_b" SEVERITY note ;

  sig_c <= sig_a AND NOT sig_b ;

END behavioral ;


