------------------------------------------------------------------------------
--
-- Filename:                register.vhd
-- Inhalt:                  verschiedene Registermodellalternativen mit
--                          GENERATE-Anweisung:
--                             - n_bit_register 
--                             - shift_register (Schieberegister)
--                          Modelle verwenden dff.vhd (vorher uebersetzen!)
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder, Gunther Lehmann
-- Letzte Aenderung:        07.03.94
--
------------------------------------------------------------------------------


ENTITY n_bit_register IS

  GENERIC (n : IN positive := 4);                              -- n Flip-Flops

  PORT (clk     : IN  bit;
        reg_in  : IN  bit_vector(n-1 DOWNTO 0);
        reg_out : OUT bit_vector(n-1 DOWNTO 0) );   

END n_bit_register;

------------------------------------------------------------------------------

ARCHITECTURE structural OF n_bit_register IS

  COMPONENT d_ff_socket
    PORT (d, clk : IN bit; q : OUT bit) ;
  END COMPONENT;

BEGIN

  reg : FOR i IN n-1 DOWNTO 0 GENERATE

    d_ff_instance: d_ff_socket                       -- n-fache Instantiierung
      PORT MAP (reg_in(i), clk, reg_out(i)) ;

  END GENERATE ;

END structural ;

------------------------------------------------------------------------------

CONFIGURATION n_bit_register_config OF n_bit_register IS

  FOR structural

    FOR reg
      FOR ALL : d_ff_socket USE ENTITY work.dff(behavioral);
      END FOR;
    END FOR;

  END FOR;

END n_bit_register_config;



------------------------------------------------------------------------------

ENTITY shift_register IS                                   -- Schieberregister

  GENERIC (n : IN positive RANGE 2 TO 64 := 4);                -- n Flip-Flops

  PORT (clk, ser_in : IN  bit;
        ser_out :     OUT bit );   

END shift_register;

------------------------------------------------------------------------------

ARCHITECTURE structural OF shift_register IS

  SIGNAL intern : bit_vector(1 TO n-1);

  COMPONENT d_ff_socket
    PORT (d, clk : IN bit; q : OUT bit) ;
  END COMPONENT;

BEGIN

  reg : FOR i IN n DOWNTO 1 GENERATE

    reg_begin: IF i = 1 GENERATE                            -- erstes Flip-Flop
      d_ff_begin: d_ff_socket
        PORT MAP (ser_in, clk, intern(i)) ;
    END GENERATE ;

    reg_middle: IF i > 1 AND i < n GENERATE              -- mittlere Flip-Flops
      d_ff_middle: d_ff_socket
        PORT MAP (intern(i-1), clk, intern(i)) ;
    END GENERATE ;

    reg_end: IF i = n GENERATE                             -- letztes Flip-Flop
      d_ff_end: d_ff_socket
        PORT MAP (intern(i-1), clk, ser_out) ;
    END GENERATE ;

  END GENERATE ;

END structural ;

------------------------------------------------------------------------------

CONFIGURATION shift_register_config OF shift_register IS

  FOR structural

    FOR reg

      FOR reg_begin
        FOR ALL : d_ff_socket USE ENTITY work.dff(behavioral); END FOR;
      END FOR;

      FOR reg_middle
        FOR ALL : d_ff_socket USE ENTITY work.dff(behavioral); END FOR;
      END FOR;

      FOR reg_end
        FOR ALL : d_ff_socket USE ENTITY work.dff(behavioral); END FOR;
      END FOR;

    END FOR;

  END FOR;

END shift_register_config;

