------------------------------------------------------------------------------
--
-- Filename:             typ_attr.vhd
-- Inhalt:               Beispiel zur Verwendung von typbezogenen Attributen. 
--                       Ergebnisse werden im File "attr_erg" abgelegt.
--
------------------------------------------------------------------------------
--
-- Author:               Bernhard Wunder  
-- Letzte Aenderung:     25.01.1994
--
------------------------------------------------------------------------------

USE        std.textio.ALL;

------------------------------------------------------------------------------

ENTITY typ_attr IS

END typ_attr;

------------------------------------------------------------------------------

ARCHITECTURE structural OF typ_attr IS

  File attr_erg : text IS OUT "attr_erg";

  SUBTYPE    asc_int       IS integer RANGE 2 TO 8;      -- aufsteigend
  SUBTYPE    desc_int      IS integer RANGE 8 DOWNTO 2;  -- abfallend

  TYPE    colors        IS (white, red, yellow, green, blue);
  SUBTYPE signal_colors IS colors RANGE red TO green;


  FUNCTION c_TO_String ( s : colors ) RETURN string IS

    VARIABLE result : string (1 TO 10) := "don't know";

  BEGIN

    CASE s IS
      WHEN white  => result := "white     ";
      WHEN red    => result := "red       ";
      WHEN yellow => result := "yellow    ";
      WHEN green  => result := "green     ";
      WHEN blue   => result := "blue      ";				
      WHEN OTHERS => result := "don't know";
    END CASE;
    RETURN result;

  END c_TO_String;


BEGIN

  PROCESS

	VARIABLE lin    :LINE;
        VARIABLE i_erg  :INTEGER;
        VARIABLE t_erg  :TIME;
        VARIABLE r_erg  :REAL;
	VARIABLE a_erg  :asc_int;
        VARIABLE d_erg  :desc_int;
        VARIABLE c_erg  :colors;
        VARIABLE sc_erg :signal_colors;

  BEGIN

	WRITE (lin, string'("Ergebnisse der typbezogenen Attribute"));
        WRITELINE (attr_erg, lin);
	WRITE (lin, string'("====================================="));
        WRITELINE (attr_erg, lin);

  --  verschiedene Beispiele vgl. entsprechendes Kapitel im Teil B

        a_erg := asc_int'LEFT;                -- erwartetes Ergebnis: 2
	WRITE (lin, string'("Attribut asc_int'LEFT             : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'RIGHT;               -- erwartetes Ergebnis: 8
	WRITE (lin, string'("Attribut asc_int'RIGHT            : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'LOW;                 -- erwartetes Ergebnis: 2
	WRITE (lin, string'("Attribut asc_int'LOW              : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'HIGH;                -- erwartetes Ergebnis: 8
	WRITE (lin, string'("Attribut asc_int'HIGH             : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);
         
        d_erg := desc_int'LEFT;               -- erwartetes Ergebnis: 8
	WRITE (lin, string'("Attribut desc_int'LEFT            : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'RIGHT;              -- erwartetes Ergebnis: 2
	WRITE (lin, string'("Attribut desc_int'RIGHT           : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'LOW;                -- erwartetes Ergebnis: 2
	WRITE (lin, string'("Attribut desc_int'LOW             : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'HIGH;               -- erwartetes Ergebnis: 8
	WRITE (lin, string'("Attribut desc_int'HIGH            : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'PRED(7);             -- erwartetes Ergebnis: 6
	WRITE (lin, string'("Attribut asc_int'PRED(7)          : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'SUCC(4);             -- erwartetes Ergebnis: 5
	WRITE (lin, string'("Attribut asc_int'SUCC(4)          : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'LEFTOF(7);           -- erwartetes Ergebnis: 6
	WRITE (lin, string'("Attribut asc_int'LEFTOF(7)        : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        a_erg := asc_int'RIGHTOF(4);          -- erwartetes Ergebnis: 5
	WRITE (lin, string'("Attribut asc_int'RIGHTOF(4)       : "));
	WRITE (lin, a_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'PRED(7);            -- erwartetes Ergebnis: 6
	WRITE (lin, string'("Attribut desc_int'PRED(7)         : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'SUCC(4);            -- erwartetes Ergebnis: 5
	WRITE (lin, string'("Attribut desc_int'SUCC(4)         : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'LEFTOF(7);          -- erwartetes Ergebnis: 8
	WRITE (lin, string'("Attribut desc_int'LEFTOF(7)       : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        d_erg := desc_int'RIGHTOF(4);         -- erwartetes Ergebnis: 3
	WRITE (lin, string'("Attribut desc_int'RIGHTOF(4)      : "));
        WRITE (lin, d_erg);
        WRITELINE (attr_erg, lin);

        sc_erg := signal_colors'RIGHT;        -- erwartetes Ergebnis: green
	WRITE (lin, string'("Attribut signal_colors'RIGHT      : "));
        WRITE (lin, c_TO_String(sc_erg));
        WRITELINE (attr_erg, lin);

        c_erg := signal_colors'BASE'RIGHT;   -- erwartetes Ergebnis: blue
	WRITE (lin, string'("Attribut signal_colors'BASE'RIGHT : "));
        WRITE (lin, c_TO_String(c_erg));
        WRITELINE (attr_erg, lin);

        c_erg := colors'LEFTOF(red);          -- erwartetes Ergebnis: white
	WRITE (lin, string'("Attribut colors'LEFTOF(red)       : "));
        WRITE (lin, c_TO_String(c_erg));
        WRITELINE (attr_erg, lin);    

        c_erg := colors'RIGHTOF(red);         -- erwartetes Ergebnis: yellow
	WRITE (lin, string'("Attribut colors'RIGHTOF(red)      : "));
        WRITE (lin, c_TO_String(c_erg));
        WRITELINE (attr_erg, lin);
       
        i_erg := colors'POS(red);             -- erwartetes Ergebnis: 1
	WRITE (lin, string'("Attribut colors'POS(red)          : "));
        WRITE (lin, i_erg);
        WRITELINE (attr_erg, lin);
        
        c_erg := colors'VAL(2);               -- erwartetes Ergebnis: yellow
	WRITE (lin, string'("Attribut colors'VAL(2)            : "));
        WRITE (lin, c_TO_String(c_erg));
        WRITELINE (attr_erg, lin);

  -- Test einiger systemabhaengiger Attribute (Bereichsgrenzen von vordefinierten Typen)

	WRITE (lin, string'(""));
        WRITELINE (attr_erg, lin);

	WRITE (lin, string'("Test einiger systemabhaengiger Attribute (Bereichsgrenzen)"));
        WRITELINE (attr_erg, lin);

	WRITE (lin, string'("=========================================================="));
        WRITELINE (attr_erg, lin);
     
        i_erg := INTEGER'LEFT;
	WRITE (lin, string'("Linke  Bereichsgrenze von integer : "));
        WRITE (lin, i_erg);
        WRITELINE (attr_erg, lin);

        i_erg := INTEGER'RIGHT;
	WRITE (lin, string'("Rechte Bereichsgrenze von integer : "));
        WRITE (lin, i_erg);
        WRITELINE (attr_erg, lin);
     
        r_erg := REAL'LEFT;
	WRITE (lin, string'("Linke  Bereichsgrenze von real    : "));
        WRITE (lin, r_erg);
        WRITELINE (attr_erg, lin);

        r_erg := REAL'RIGHT;
	WRITE (lin, string'("Rechte Bereichsgrenze von real    : "));
        WRITE (lin, r_erg);
        WRITELINE (attr_erg, lin);

        WAIT;

  END PROCESS;

END structural;

------------------------------------------------------------------------------
