------------------------------------------------------------------------------
--
-- Filename:                or2.vhd
-- Inhalt:                  Zeitabhaengiges Modell eines OR2-Gatters, welches
--                          die 9-wertige Logik std_logic_1164 verwendet
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder
-- Letzte Aenderung:        18.01.1994
--
------------------------------------------------------------------------------

LIBRARY    ieee;
USE        ieee.std_logic_1164.all;

------------------------------------------------------------------------------

ENTITY or2 IS

    GENERIC (tp_lh: time := 170 ps; 
             tp_hl: time := 230 ps);
    PORT (a,b:  IN  std_ulogic;
          y:    OUT std_ulogic);

END or2;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF or2 IS

-- Funktion zur Bestimmung des Minimums zweier Zeiten

    FUNCTION min (x, y: time) RETURN time IS
    BEGIN
        IF y < x THEN
            RETURN y;
        END IF;
        RETURN x;
    END min;

BEGIN

-- Berechnung des Ausgangs nach entsprechender Verzoegerung

    y <= '1' AFTER tp_lh WHEN ((a OR b) = '1') ELSE
         '0' AFTER tp_hl WHEN ((a OR b) = '0') ELSE
         'X' AFTER min (tp_lh,tp_hl);

END behavioral;

------------------------------------------------------------------------------

CONFIGURATION or2_config OF or2 IS

    FOR behavioral
    END FOR;

END or2_config;
