------------------------------------------------------------------------------
--
-- Filename:              halfadd.vhd
-- Inhalt:                Modell eines Halbaddierers, realisiert 
--                        mit verschiedenen Architekturen:
--
--                        -- ARCHITECTURE  "behavioral_seq" (sequentiell)
--                           CONFIGURATION "halfadder1"
--
--                        -- ARCHITECTURE  "behavioral_par" (nebenlaeufig)
--                           CONFIGURATION "halfadder2" 
--
--                        -- ARCHITECTURE  "structural"
--                           CONFIGURATION "halfadder3"
--
------------------------------------------------------------------------------
--
-- Autor:                Bernhard Wunder  
-- Letzte Aenderung:     04.03.1994
--
------------------------------------------------------------------------------

ENTITY halfadder IS 

   PORT (sum_a, sum_b: IN bit; sum, carry: OUT bit);  

END halfadder ;

------------------------------------------------------------------------------
------------------------------------------------------------------------------

ARCHITECTURE behavioral_seq OF halfadder IS

BEGIN

    PROCESS (sum_a, sum_b)

    BEGIN
	IF (sum_a = '1' AND sum_b = '1' ) THEN
           sum <= '0';
           carry <= '1';
        ELSE
           IF ( sum_a = '1' OR sum_b = '1' ) THEN
              sum <= '1';
           ELSE
              sum <= '0';
           END IF;
           carry <= '0';
        END IF;
    END PROCESS;

END behavioral_seq;

--------------------------------------------------------------------------------

ARCHITECTURE behavioral_par OF halfadder IS

BEGIN

   sum   <= sum_a XOR sum_b;
   carry <= sum_a AND sum_b;

END behavioral_par;

--------------------------------------------------------------------------------

ARCHITECTURE structural OF halfadder IS

   COMPONENT xor2
      PORT (c1, c2: IN bit; c3: OUT bit);
   END COMPONENT;

   COMPONENT and2
      PORT (c4, c5: IN bit; c6: OUT bit);
   END COMPONENT;

BEGIN

  xor_instance : xor2 PORT MAP (sum_a, sum_b, sum) ;
  and_instance : and2 PORT MAP (sum_a, sum_b, carry) ;
  
END structural;

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

CONFIGURATION halfadder1_config OF halfadder IS

    FOR behavioral_seq                 
    END FOR;     

END halfadder1_config;

--------------------------------------------------------------------------------
                  
CONFIGURATION halfadder2_config OF halfadder IS

    FOR behavioral_par    
    END FOR;                     

END halfadder2_config;

--------------------------------------------------------------------------------

CONFIGURATION halfadder3_config OF halfadder IS

    FOR structural
        FOR xor_instance: xor2 USE ENTITY work.exor (behavioral)
          PORT MAP(c1,c2,c3);
        END FOR;
        FOR and_instance: and2 USE ENTITY work.and2 (behavioral)
          PORT MAP (c4,c5,c6);
        END FOR;                 
    END FOR;                     

END halfadder3_config;

