------------------------------------------------------------------------------
--
-- Filename:                alu_tb.vhd
-- Inhalt:                  Testbench fuer das ALU-Verhaltensmodell
--
------------------------------------------------------------------------------
--
-- Autor:                   Bernhard Wunder  
-- Letzte Aenderung:        20.01.1994
--
------------------------------------------------------------------------------

USE work.alu_pack.all;
USE std.textio.all;

------------------------------------------------------------------------------

ENTITY alu_tb IS

END alu_tb ;

------------------------------------------------------------------------------

ARCHITECTURE behavioral OF alu_tb IS

    COMPONENT alu_socket 
        PORT (instruction          : IN command;
              operand_a, operand_b : IN parity_8;
              result               : OUT parity_16);
    END COMPONENT;

    SIGNAL cmd : command ;
    SIGNAL op_a, op_b : parity_8;
    SIGNAL res : parity_16;

BEGIN

--------------------------------------
--  Instantiiere "Model under test"
--------------------------------------

    mut : alu_socket
          PORT MAP (cmd,op_a,op_b,res);


    command_execution : PROCESS

        FILE input_file  : text IS IN  input_filename;
        FILE output_file : text IS OUT output_filename;

        VARIABLE lin     : line;
        VARIABLE good    : boolean;
        VARIABLE cmd_str : string (1 TO 6);
        VARIABLE bit_9   : bit_vector(8 DOWNTO 0);      -- bit_9(8)=parity
        VARIABLE space   : character;

    BEGIN
 
--------------------------------------
--      Lese Befehl und Operanden
--------------------------------------

        WHILE ENDFILE (input_file) = false LOOP
            READLINE (input_file,lin);
            READ (lin,cmd_str,good);
            ASSERT good REPORT "Error reading file" SEVERITY failure;
            cmd <= str_to_command(cmd_str);

            READ (lin,space,good);
            ASSERT good REPORT "Error reading file" SEVERITY failure;

            READ (lin,bit_9,good);
            ASSERT good REPORT "Error reading file" SEVERITY failure;
            op_a.vector <= bit_9(7 DOWNTO 0);
            op_a.parity <= bit_9(8);

            READ (lin,space,good);
            ASSERT good REPORT "Error reading file" SEVERITY failure;

            READ (lin,bit_9,good);
            ASSERT good REPORT "Error reading file" SEVERITY failure;
            op_b.vector <= bit_9(7 DOWNTO 0);
            op_b.parity <= bit_9(8);

            WAIT FOR 10 ms;

--------------------------------------
--         Schreibe Ergebnis
--------------------------------------
            WRITE (lin, res.parity);
            WRITE (lin, ' ');
            WRITE (lin, res.vector);
            WRITELINE (output_file, lin);

        END LOOP;

        WAIT;

    END PROCESS;

END behavioral;

------------------------------------------------------------------------------

CONFIGURATION alu_tb_conf OF alu_tb IS

    FOR behavioral
        FOR mut : alu_socket
            USE ENTITY work.alu (behavioral)
                PORT MAP(instruction,operand_a,operand_b,result);
        END FOR;
    END FOR;

END alu_tb_conf;

  
